/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 *   Justin Bradford <jab@atdot.org> (original author of nsDigestAuth.h)
 *   An-Cheng Huang <pach@cs.cmu.edu>
 */

#ifndef nsDigestAuth_h__
#define nsDigestAuth_h__

#include "nsIHttpAuthenticator.h"
#include "nsIURI.h"
#include "nsString.h"
#include "nsCOMPtr.h"
#include "nsISignatureVerifier.h"

#define ALGO_SPECIFIED 0x01
#define ALGO_MD5 0x02
#define ALGO_MD5_SESS 0x04
#define QOP_AUTH 0x01
#define QOP_AUTH_INT 0x02

#define DIGEST_LENGTH 16
#define EXPANDED_DIGEST_LENGTH 32
#define NONCE_COUNT_LENGTH 8

//class nsIURI;

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

class nsHttpDigestAuth : public nsIHttpAuthenticator
{
  public:
    NS_DECL_ISUPPORTS
    NS_DECL_NSIHTTPAUTHENTICATOR

    nsHttpDigestAuth();
    virtual ~nsHttpDigestAuth();

  protected:
    nsresult ExpandToHex(const char * digest, char * result);
    nsresult CalculateResponse(const char * ha1_digest,
        const char * ha2_digest,
        nsCAutoString & nonce,
        PRUint16 qop,
        const char * nonce_count,
        nsCAutoString & cnonce,
        char * result);
    nsresult CalculateHA1(nsCAutoString & username,
        nsCAutoString & password,
        nsCAutoString & realm,
        PRUint16 algorithm,
        nsCAutoString & nonce,
        nsCAutoString & cnonce,
        char * result);
    nsresult CalculateHA2(const char * http_method,
        const char * http_uri,
        PRUint16 qop,
        const char * body_digest,
        char * result);
    nsresult ParseChallenge(const char * challenge,
        nsCAutoString & realm,
        nsCAutoString & domain,
        nsCAutoString & nonce,
        nsCAutoString & opaque,
        PRBool * stale,
        PRUint16 * algorithm,
        PRUint16 * qop);

    nsCOMPtr<nsISignatureVerifier> mVerifier;
    char mHashBuf[DIGEST_LENGTH];
    PRBool gotVerifier;

    // result is in mHashBuf
    nsresult MD5Hash(char *buf, PRUint32 len);

};

#endif // nsHttpDigestAuth_h__
